/*
 * Copyright 2018 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package com.google.android.overtheair.v1;

import com.google.common.base.Preconditions;
import com.google.common.collect.ImmutableMap;
import com.google.api.pathtemplate.PathTemplate;
import com.google.api.resourcenames.ResourceName;
import java.util.Map;
import java.util.ArrayList;
import java.util.List;

// AUTO-GENERATED DOCUMENTATION AND CLASS
@javax.annotation.Generated("by GAPIC protoc plugin")
public class PackagePathName implements ResourceName {

  private static final PathTemplate PATH_TEMPLATE =
      PathTemplate.createWithoutUrlEncoding("deployments/{deployment}/packages/{package_path=**}");

  private volatile Map<String, String> fieldValuesMap;

  private final String deployment;
  private final String packagePath;

  public String getDeployment() {
    return deployment;
  }

  public String getPackagePath() {
    return packagePath;
  }

  public static Builder newBuilder() {
    return new Builder();
  }

  public Builder toBuilder() {
    return new Builder(this);
  }

  private PackagePathName(Builder builder) {
    deployment = Preconditions.checkNotNull(builder.getDeployment());
    packagePath = Preconditions.checkNotNull(builder.getPackagePath());
  }

  public static PackagePathName of(String deployment, String packagePath) {
    return newBuilder()
      .setDeployment(deployment)
      .setPackagePath(packagePath)
      .build();
  }

  public static String format(String deployment, String packagePath) {
    return newBuilder()
      .setDeployment(deployment)
      .setPackagePath(packagePath)
      .build()
      .toString();
  }

  public static PackagePathName parse(String formattedString) {
    if (formattedString.isEmpty()) {
      return null;
    }
    Map<String, String> matchMap =
        PATH_TEMPLATE.validatedMatch(formattedString, "PackagePathName.parse: formattedString not in valid format");
    return of(matchMap.get("deployment"), matchMap.get("package_path"));
  }

  public static List<PackagePathName> parseList(List<String> formattedStrings) {
    List<PackagePathName> list = new ArrayList<>(formattedStrings.size());
    for (String formattedString : formattedStrings) {
      list.add(parse(formattedString));
    }
    return list;
  }

  public static List<String> toStringList(List<PackagePathName> values) {
    List<String> list = new ArrayList<String>(values.size());
    for (PackagePathName value : values) {
      if (value == null) {
        list.add("");
      } else {
        list.add(value.toString());
      }
    }
    return list;
  }

  public static boolean isParsableFrom(String formattedString) {
    return PATH_TEMPLATE.matches(formattedString);
  }

  public Map<String, String> getFieldValuesMap() {
    if (fieldValuesMap == null) {
      synchronized (this) {
        if (fieldValuesMap == null) {
          ImmutableMap.Builder<String, String> fieldMapBuilder = ImmutableMap.builder();
          fieldMapBuilder.put("deployment", deployment);
          fieldMapBuilder.put("packagePath", packagePath);
          fieldValuesMap = fieldMapBuilder.build();
        }
      }
    }
    return fieldValuesMap;
  }

  public String getFieldValue(String fieldName) {
    return getFieldValuesMap().get(fieldName);
  }

  @Override
  public String toString() {
    return PATH_TEMPLATE.instantiate("deployment", deployment, "package_path", packagePath);
  }

  /** Builder for PackagePathName. */
  public static class Builder {

    private String deployment;
    private String packagePath;

    public String getDeployment() {
      return deployment;
    }

    public String getPackagePath() {
      return packagePath;
    }

    public Builder setDeployment(String deployment) {
      this.deployment = deployment;
      return this;
    }

    public Builder setPackagePath(String packagePath) {
      this.packagePath = packagePath;
      return this;
    }

    private Builder() {
    }

    private Builder(PackagePathName packagePathName) {
      deployment = packagePathName.deployment;
      packagePath = packagePathName.packagePath;
    }

    public PackagePathName build() {
      return new PackagePathName(this);
    }
  }

  @Override
  public boolean equals(Object o) {
    if (o == this) {
      return true;
    }
    if (o instanceof PackagePathName) {
      PackagePathName that = (PackagePathName) o;
      return (this.deployment.equals(that.deployment))
          && (this.packagePath.equals(that.packagePath));
    }
    return false;
  }

  @Override
  public int hashCode() {
    int h = 1;
    h *= 1000003;
    h ^= deployment.hashCode();
    h *= 1000003;
    h ^= packagePath.hashCode();
    return h;
  }
}

